/**
* DevExtreme (ui/file_manager.d.ts)
* Version: 20.1.2
* Build date: Mon Apr 06 2020
*
* Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import '../jquery_augmentation';

import {
    dxElement
} from '../core/element';

import {
    event
} from '../events/index';

import FileSystemItem from '../file_management/file_system_item';

import {
    dxContextMenuItem
} from './context_menu';

import {
    dxDataGridColumn
} from './data_grid';

import {
    dxToolbarItem
} from './toolbar';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerOptions extends WidgetOptions<dxFileManager> {
   /** Specifies the allowed upload file extensions. */
    allowedFileExtensions?: Array<string>;
   /** Configures the context menu settings. */
    contextMenu?: dxFileManagerContextMenu;
   /** Specifies the path that is used when the FileManager is initialized. */
    currentPath?: string;
   /** Specifies an array of path keys to the current location. */
    currentPathKeys?: Array<string>;
   /** Customizes columns in details view. Applies only if itemView.mode is "details". */
    customizeDetailColumns?: ((columns: Array<dxDataGridColumn>) => Array<dxDataGridColumn>);
   /** Allows you to provide custom icons to be used as thumbnails. */
    customizeThumbnail?: ((fileSystemItem: FileSystemItem) => string);
   /** Specifies the file system provider. */
    fileSystemProvider?: any;
   /** Configures the file and folder view. */
    itemView?: {details?: { columns?: Array<dxFileManagerDetailsColumn | string>}, mode?: 'details' | 'thumbnails', showFolders?: boolean, showParentFolder?: boolean };
   /**  */
    onContextMenuItemClick?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, itemData?: any, itemElement?: dxElement, itemIndex?: number, event?: event }) => any);
   /** A function that is executed when the current directory is changed. */
    onCurrentDirectoryChanged?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, directory?: FileSystemItem }) => any);
   /** A function that is executed when the selected file is opened. */
    onSelectedFileOpened?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, file?: FileSystemItem }) => any);
   /** A function that is executed when a file system item is selected or selection is canceled. */
    onSelectionChanged?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, currentSelectedItemKeys?: Array<string>, currentDeselectedItemKeys?: Array<string>, selectedItems?: Array<FileSystemItem>, selectedItemKeys?: Array<string>}) => any);
   /**  */
    onToolbarItemClick?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, itemData?: any, itemElement?: dxElement, itemIndex?: number, event?: event }) => any);
   /**  */
    onFocusedItemChanged?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, item?: FileSystemItem, itemElement?: dxElement }) => any);
   /**  */
    onErrorOccurred?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, errorCode?: number, errorText?: string, fileSystemItem?: FileSystemItem }) => any);
   /** Specifies actions that a user is allowed to perform on files and folders. */
    permissions?: { copy?: boolean, create?: boolean, download?: boolean, move?: boolean, remove?: boolean, rename?: boolean, upload?: boolean };
   /** Specifies the root folder name. */
    rootFolderName?: string;
   /** Specifies whether a user can select a single or multiple files and folders in the item view simultaneously. */
    selectionMode?: 'multiple' | 'single';
   /**  */
    selectedItemKeys?: Array<string>;
   /**  */
    focusedItemKey?: string;
   /** Configures toolbar settings. */
    toolbar?: dxFileManagerToolbar;
   /** Configures upload settings. */
    upload?: { maxFileSize?: number, chunkSize?: number };
}/** [tags] ctp The FileManager is a widget that allows users to upload, select, and manage files and directories in different file storages. #include common-ctp-note with { component: "FileManager" } */
export default class dxFileManager extends Widget {
    constructor(element: Element, options?: dxFileManagerOptions)
    constructor(element: JQuery, options?: dxFileManagerOptions)
   /** Gets the current directory object. */
    getCurrentDirectory(): any;
   /** Gets the selected items. */
    getSelectedItems(): Array<any>;
   /** Reloads data and repaints the widget. */
    refresh(): Promise<any> & JQueryPromise<any>;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerContextMenu {
   /** Configures context menu items' settings. */
    items?: Array<dxFileManagerContextMenuItem | 'create' | 'upload' | 'refresh' | 'download' | 'move' | 'copy' | 'rename' | 'delete'>;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerContextMenuItem extends dxContextMenuItem {
   /** Configures settings of a context menu item's subitems. */
    items?: Array<dxFileManagerContextMenuItem>;
   /** Specifies the context menu item's name. */
    name?: 'create' | 'upload' | 'refresh' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | string;
   /** Specifies the context menu item's visibility. */
    visible?: boolean;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerToolbar {
   /** Configures settings of the toolbar items that are visible when users select files. */
    fileSelectionItems?: Array<dxFileManagerToolbarItem | 'showNavPane' | 'create' | 'upload' | 'refresh' | 'switchView' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | 'clear' | 'separator'>;
   /** Configures toolbar items' settings. */
    items?: Array<dxFileManagerToolbarItem | 'showNavPane' | 'create' | 'upload' | 'refresh' | 'switchView' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | 'clear' | 'separator'>;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerToolbarItem extends dxToolbarItem {
   /** Specifies the toolbar item's location. */
    location?: 'after' | 'before' | 'center';
   /** Specifies the toolbar item's name. */
    name?: 'showNavPane' | 'create' | 'upload' | 'refresh' | 'switchView' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | 'clear' | 'separator' | string;
   /** Specifies the toolbar item's visibility. */
    visible?: boolean;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerDetailsColumn {
   /**  */
    alignment?: 'center' | 'left' | 'right' | undefined;
   /**  */
    caption?: string;
   /**  */
    dataField?: string;
   /**  */
    hidingPriority?: number;
   /**  */
    sortIndex?: number;
   /**  */
    sortOrder?: 'asc' | 'desc' | undefined;
   /**  */
    visible?: boolean;
   /**  */
    width?: number | string;
}

declare global {
interface JQuery {
    dxFileManager(): JQuery;
    dxFileManager(options: "instance"): dxFileManager;
    dxFileManager(options: string): any;
    dxFileManager(options: string, ...params: any[]): any;
    dxFileManager(options: dxFileManagerOptions): JQuery;
}
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type Options = dxFileManagerOptions;

/** @deprecated use Options instead */
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type IOptions = dxFileManagerOptions;